// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
{**
   Implementation of IIOPServerInvokeRecord and IIOPServer.
 }
unit iiop;

interface

{$I dorb.inc}

{$IFNDEF USELIB}

uses
  iiop_int,orb_int,orbtypes,orb,giop_int,addr_int,Classes,req_int,map_int,
  env_int,code_int,osthread,mtmanager_int,iop,except_int;

type

  TIIOPServerInvokeRecord = class(TInterfacedObject,IIIOPServerInvokeRecord)
  private
    FConnection: IGIOPConnection;
    FORBId : ORBMessageID;
    FREQId : MessageID;
    FRequest : IORBRequest;
    FObject: IORBObject;
    FActive: boolean;
    FOrbMsgId: MessageID;
  protected
    procedure init_invoke (const conn: IGIOPConnection; reqid: MessageID;
      orbid: ORBMessageID; const req : IORBRequest; const obj: IORBObject);
    function get_request(): IORBRequest;
    function orbmsgid(): MessageID;
    function orbid: ORBMessageID;
    function reqid: MessageID;
    procedure set_orbid(oid: ORBMessageID);
    procedure set_reqid(reqid: MessageID);
    function connection : IGIOPConnection;
    procedure clean();
    function active(): boolean;
    procedure deactivate();
  public
  end;

  TIIOPServer = class(TORBObject,IIIOPServer,IObjectAdaptor,ITransportServerCallback,IGIOPCallBack,IORBCallback)
  private
    FIIOPVersion: _ushort;
    FMaxMessageSize: _ulong;
    FTServers : IInterfaceList;
{$IFDEF USE_IOP_CACHE}
    FCacheRecord : IIIOPServerInvokeRecord;
    FCacheUsed: Boolean;
{$ENDIF}
    FConnections : IInterfaceList;
    FORBIds : ILongInterfaceMap;
    //FORBIdsLock: TMutex;
    FConnLock: TMutex;
    FConnMgr: TGIOPConnectionMgr;
    FMonitor: TThreadMonitor;
    procedure deref_connection(const conn: IGIOPConnection; all: boolean = false);
  protected
    function skeleton(const obj: IORBObject): IORBObject;
    function is_local(): Boolean;
    procedure add_invoke(rec: IIIOPServerInvokeRecord);
    procedure del_invoke_orbid(rec : IIIOPServerInvokeRecord);
    function pull_invoke_orbid(orbid: ORBMessageID):IIIOPServerInvokeRecord;
    procedure del_invoke_reqid(orbid: MessageID;const conn: IGIOPConnection);
    procedure kill_connection(const conn: IGIOPConnection; redo: boolean = false);
    function handle_input(const conn: IGIOPConnection; const inp: IBuffer): Boolean;
    function exec_invoke_request(const inc: IGIOPInContext;obj : IORBObject;
      req: IORBRequest;  const pr: IPrincipal; resp_exp: Boolean;const conn: IGIOPConnection; id: ORBMessageID): ORBMessageID;
    function handle_invoke_request (const conn: IGIOPConnection;const inc: IGIOPInContext):Boolean;
    function handle_locate_request (const conn: IGIOPConnection;const inc: IGIOPInContext):Boolean;
    function handle_cancel_request (const conn: IGIOPConnection;const inc: IGIOPInContext):Boolean;
    procedure handle_invoke_reply (id: ORBMessageID);
    procedure handle_locate_reply (msgid: MessageID);
    procedure handle_bind_reply (id: ORBMessageID);
    function listen(): Boolean;
    function listen_addr(addr: IAddress;addr2: IAddress):Boolean;
    function has_object(const obj: IORBObject):Boolean;
    function waitfor(const orb: IORB; id: ORBMessageID; event: TORBCallbackEvent; tmout: long = -1): boolean;
    procedure notify(const orb: IORB; id: ORBMessageID; evt : TORBCallbackEvent);
    procedure ts_callback(ts: ITransportServer;event :TTransportServerCallBackEvent);
    //procedure callback (const orb: IORB; mid: MessageID; evt : TORBCallbackEvent);
    function invoke(mid: ORBMessageID;const obj : IORBObject; const req : IORBRequest; const pr: IPrincipal; response_exp: Boolean): Boolean; stdcall;
    function bind(mid: ORBMessageID; const repoid: string; const oid: IObjectTag; const addr: IAddress): Boolean;
    function get_oaid(): ObjectID;
    function giop_callback(const conn: IGIOPConnection;event: TEventType): Boolean;
    function input_giop_callback(const conn: IGIOPConnection;const buf: IBuffer{;event: TEventType}): Boolean;
    function create_invoke(): IIIOPServerInvokeRecord;
    procedure answer_invoke(mid: ORBMessageID; const obj: IORBObject; const req: IORBRequest; stat: TInvokeStatus);
    procedure shutdown(WaitForCompletion: Boolean);
    procedure conn_closed(const conn: IGIOPConnection);
    procedure cancel(mid: ORBMessageID);
    procedure abort_invoke_orbid(rec : IIIOPServerInvokeRecord);
    procedure sendORBMsg(const conn: IGIOPConnection;event: TORBMsgEvent);
  public
    constructor Create(dorb: IORB; iiop_version: _ushort = $0100; max_size: _ulong = 0);
    destructor Destroy(); override;
  end;

implementation

uses
  address,iior_int,orbcomp,mcomp_int,codec,decoder,encoder,exceptions,
  ior,map,utils,intercept,mtmanager,SysUtils;

//**************************************************************************
//  TIIOPServer
//**************************************************************************
constructor TIIOPServer.Create(dorb: IORB; iiop_version: _ushort; max_size: _ulong);
var
  value: string;
begin
  inherited Create();
  //FORBIdsLock := TMutex.Create(false, maRecursive);
  FConnLock := TMutex.Create;
  FTServers := TInterfaceList.Create;
  FConnections := TInterfaceList.Create;
  FORBIds := TLongInterfaceMap.Create;
  FConnMgr := TGIOPConnectionMgr.Create;
{$IFDEF USE_IOP_CACHE}
  FCacheUsed := false;
  FCacheRecord := TIIOPServerInvokeRecord.Create;
{$ENDIF}
  FIIOPVersion := iiop_version;
  FMaxMessageSize := max_size;
  value := FORB.get_properties().get_Property('mtdorb.orb.connections_limit');
  if value <> '' then
    FConnMgr.set_conn_limit(StrToInt(value))
  else
    FConnMgr.set_conn_limit(10);
  FMonitor := TThreadMonitor.Create;
  FORB.register_oa(self as IObjectAdaptor, OAPriorityRemote);
end;

destructor TIIOPServer.Destroy();
begin
  FConnLock.lock();
  try
    FConnections := nil;
  finally
    FConnLock.unlock();
  end; { try/finally }

  FTServers := nil;

  FConnLock.lock();
  try
    FORBIDs := nil;
  finally
    FConnLock.unlock();
  end; { try/finally }
  //FORBIdsLock.Free;
  FConnLock.Free;
  FConnMgr.Free;
  FMonitor.Free;
  inherited Destroy();
end;

procedure TIIOPServer.conn_closed(const conn: IGIOPConnection);
var
  outc : IGIOPOutContext;
begin
  outc := TGIOPOutContext.Create(conn.codec,nil);
  conn.codec.put_close_msg(outc);
  { TODO -oOVS : output and flush }
  conn.output(outc.buffer());
{$IFNDEF HAVE_THREADS}
  deref_connection(conn, true);
{$ELSE}
  conn.terminate();
{$ENDIF}
end;

procedure TIIOPServer.shutdown(WaitForCompletion: Boolean);
var
  i : integer;
  trans: ITransportServer;
  sync: ISynchronized;
begin
  sync := TSynchronized.Create(FMonitor);
  while FORBIDs.Count > 0 do
    FMonitor.wait();
  sync := nil;
  
  for i := 0 to Pred(FTServers.Count) do begin
    trans := ITransportServer(FTServers[i]);
    trans.close();
    trans.aselect(FORB.dispatcher,nil);
  end;
  FTServers.Clear;
  FConnLock.lock();
  try
    for i := 0 to Pred(FConnections.Count) do begin
      conn_closed(IGIOPConnection(FConnections[i]));
      deref_connection(IGIOPConnection(FConnections[i]));
    end;
    FConnections.Clear;
  finally
    FConnLock.unlock();
  end; { try/finally }
{$IFDEF USE_IOP_CACHE}
  if FCacheUsed then
    FORB.cancel(FCacheRecord.orbid);
{$ENDIF}
  //FORBIdsLock.lock();
  sync := TSynchronized.Create(FMonitor);
  try
    for i := Pred(FORBIDs.Count) downto 0 do
      //FORB.cancel(IIIOPServerInvokeRecord(FORBIDs[i]).orbid);
      FORB.cancel(IIIOPServerInvokeRecord(FORBIDs.Items(i)).orbid);
    FORBIDs.Clear;
  finally
    sync := nil;
    //FORBIdsLock.unlock();
  end; { try/finally }
{$IFDEF USE_IOP_CACHE}
  FCacheUsed := false;
{$ENDIF}
  FORB.answer_shutdown(self);
end;

procedure TIIOPServer.answer_invoke(mid: ORBMessageID; const obj: IORBObject; const req: IORBRequest; stat: TInvokeStatus);
begin
  assert(false);
end;

function TIIOPServer.skeleton(const obj: IORBObject): IORBObject;
begin
  result := nil;
end;

procedure TIIOPServer.deref_connection(const conn: IGIOPConnection;
  all: boolean);
begin
  if conn.deref() then begin
    sendORBMsg(conn, omeKillConn);
    FConnMgr.release_conn();
  end;
end;

procedure TIIOPServer.abort_invoke_orbid(rec : IIIOPServerInvokeRecord);
begin
  FORB.cancel(rec.orbmsgid());
  //del_invoke_orbid(mid);
end;

procedure TIIOPServer.sendORBMsg(const conn: IGIOPConnection;
  event: TORBMsgEvent);
var
  msg: IMsgType;
  orbmsg: IOrbMsg;
begin
  orbmsg := TORBMsg.Create(conn, nil, event);
  orbmsg._AddRef();
  msg := TMsgType.Create(Pointer(orbmsg));
  TPM.put_msg(odORB, msg);
end;

function TIIOPServer.is_local(): Boolean;
begin
  result := false;
end;

function TIIOPServer.get_oaid(): ObjectID;
begin
  result := 'dorb-iiop-server';
end;

function TIIOPServer.giop_callback(const conn: IGIOPConnection;event: TEventType): Boolean;
var
  addr : IAddress;
begin
  result := false;
  case event of
    orbtypes.InputReady:
      result := handle_input(conn, conn.input);
    orbtypes.Idle: begin
      conn_closed(conn);
      kill_connection(conn);
    end;
    orbtypes.Closed: begin
      //conninterceptor
      addr := conn.transport().peer();
      Assert(addr <> nil);
      TConnInterceptor._exec_client_disconnect(addr.toString);
      kill_connection(conn);
    end;
    else
      assert(false,'not implemented');
  end;
end;

function TIIOPServer.input_giop_callback(const conn: IGIOPConnection;
  const buf: IBuffer{; event: TEventType}): Boolean;
begin
  result := handle_input(conn, buf);
end;

procedure TIIOPServer.ts_callback(ts: ITransportServer;event :TTransportServerCallBackEvent);
var
  transp : ITransport;
  addr : IAddress;
  conn : IGIOPConnection;
  res: boolean;
begin
  case event of
    orbtypes.tsAccept: begin
        transp := ts.accept;
        if transp <> nil then begin
          if not FConnMgr.aquire_conn() then exit;
          if transp.bad then  exit;
          addr := transp.peer;
          //conninterceptor
          Assert(addr <> nil);
          res := TConnInterceptor._exec_client_connect(addr.toString);
          if not res then begin
            transp._Release;
          end
          else begin
            conn := TGIOPConnection.Create(FConnMgr.Dispatcher(), FORB, transp, self,
              TGIOPCodec.Create(TDecoder.Create(nil) as IDecoder, TEncoder.Create(nil) as IEncoder, FIIOPVersion) as IGIOPCodec, 0, FMaxMessageSize);
            FConnLock.lock();                                                                     
            try
              FConnections.Add(conn);
            finally
              FConnLock.unlock();
            end; { try/finally }
            conn.start();
          end;
        end;
      end;
    else
      //assert(false,'not implemented');
  end;
end;

function TIIOPServer.create_invoke(): IIIOPServerInvokeRecord;
begin
{$IFDEF USE_IOP_CACHE}
  if FCacheUsed then
    result := TIIOPServerInvokeRecord.Create
  else
    begin
      FCacheUsed := True;
      result := FCacheRecord;
    end;
{$ELSE}
  result := TIIOPServerInvokeRecord.Create;
{$ENDIF}
end;

procedure TIIOPServer.add_invoke(rec: IIIOPServerInvokeRecord);
var
  //al: IAuto_Lock;
  sync: ISynchronized;
begin
  sync := TSynchronized.Create(FMonitor);
  //al := TAuto_Lock.Create(FORBIdsLock);
{$IFDEF USE_IOP_CACHE}
  if rec <> FCacheRecord then
    FORBIDs[rec.orbmsgid] := rec;
{$ELSE}
  FORBIDs[rec.orbmsgid] := rec;
  rec._AddRef;
  FORB.set_RequestHint(rec.orbid(), Pointer(rec));
{$ENDIF}
end;

procedure TIIOPServer.notify(const orb: IORB; id: ORBMessageID;
  evt: TORBCallbackEvent);
begin
  case evt of
    _Invoke: handle_invoke_reply(id);
    _Bind: handle_bind_reply(id);
    else
      assert(false,'not implemented');
  end;
end;

function TIIOPServer.waitfor(const orb: IORB; id: ORBMessageID;
  event: TORBCallbackEvent; tmout: Integer): boolean;
begin
  Assert(false);
  result := true;
end;

function TIIOPServer.pull_invoke_orbid(orbid: ORBMessageID):IIIOPServerInvokeRecord;
var
  //al: IAuto_Lock;
  sync: ISynchronized;
begin
  sync := TSynchronized.Create(FMonitor);
  result := nil;
{$IFDEF USE_IOP_CACHE}
  if FCacheUsed and (FCacheRecord.orbid = orbid) then
    result := FCacheRecord
  else
    result := FORBIDs[orbid] as IIIOPServerInvokeRecord;
{$ENDIF}
  //al := TAuto_Lock.Create(FORBIdsLock);
  result := IIIOPServerInvokeRecord(FORB.get_RequestHint(orbid));
  result._Release;
  if (result <> nil) and result.active() then begin
    result.deactivate();
    Exit;
  end;
  del_invoke_orbid(result);
end;

procedure TIIOPServer.del_invoke_orbid(rec : IIIOPServerInvokeRecord);
var
  //al: IAuto_Lock;
  sync: ISynchronized;
begin
  sync := TSynchronized.Create(FMonitor);
  //al := TAuto_Lock.Create(FORBIdsLock);
  Assert(not rec.active());
{$IFDEF USE_IOP_CACHE}
  if FCacheUsed and (FCacheRecord.orbid = orbid) then
    begin
      FCacheRecord.clean;
      FCacheUsed := false;
      exit;
    end;
{$ELSE}
  deref_connection(rec.connection());
  if rec <> nil then
    begin
      FORBIDs.erase(rec.orbmsgid());
      rec.clean;
    end;
{$ENDIF}
  FMonitor.notifyAll();
end;

procedure TIIOPServer.del_invoke_reqid(orbid: MessageID;
  const conn: IGIOPConnection);
begin
  assert(false,'not implemented');
end;

function TIIOPServer.exec_invoke_request(const inc: IGIOPInContext;
  obj: IORBObject; req: IORBRequest;  const pr: IPrincipal; resp_exp: Boolean;
  const conn: IGIOPConnection; id: ORBMessageID): ORBMessageID;
var
  rep : string;
  tag : IObjectTag;
begin
  if req.op_name = '_bind' then
    begin
      conn.codec.get_bind_request(inc,rep,tag);
      result := Forb.bind_async(rep,tag,nil,self,id);
    end
  else
    result := Forb.invoke_async(obj,req,pr,resp_exp,self,id);
end;

procedure TIIOPServer.handle_bind_reply(id: ORBMessageID);
var
  obj : IORBObject;
  stat: TLocateStatus;
  rec : IIIOPServerInvokeRecord;
  giop_stat : TLocateStatusType;
  outc: IGIOPOutContext;
begin
  rec := pull_invoke_orbid(id);
  stat := forb.get_bind_reply(id,obj);
  if rec = nil then exit;
  giop_stat := OBJECT_HERE;
  case stat of
    LocateUnknown: giop_stat := UNKNOWN_OBJECT;
    LocateHere : giop_stat := OBJECT_HERE;
    LocateForward : giop_stat := OBJECT_FORWARD;
  end;
  outc := TGIOPOutContext.Create(rec.connection.codec);
  rec.connection.codec.put_bind_reply(outc,rec.reqid,giop_stat,obj);
  rec.connection.output(outc.buffer);
  del_invoke_orbid(rec);
end;

function TIIOPServer.handle_cancel_request(const conn: IGIOPConnection;
  const inc: IGIOPInContext): Boolean;
begin
  result := false;
  assert(false,'not implemented');
end;

function TIIOPServer.handle_input(const conn: IGIOPConnection; const inp: IBuffer): Boolean;
var
  inc : IGIOPInContext;
  size: _ulong;
  flags: byte;
  mt: TMessageType;
begin
  result := false;
  inc := TGIOPInContext.Create(conn.codec,inp,nil);
  inc.set_converter(conn.codec().converter());
  if not conn.codec.get_header(inc,mt,size,flags) then exit;
  case mt of
    orbtypes.Request: result := handle_invoke_request(conn,inc);
    // for some strange reason Orbacus 4 clients send CloseConnection
    // upon exit ...
    orbtypes.CloseConnection: ;
    else begin
      conn.activeDeref();
      assert(false,'not implemented');
    end;
  end;
end;

procedure TIIOPServer.handle_invoke_reply(id: ORBMessageID);
var
  obj : IORBObject;
  stat : TInvokeStatus;
  req: IORBRequest;
  rec: IIIOPServerInvokeRecord;
  gstat : TReplyStatusType;
  outc: IGIOPOutContext;
  ad: AddressingDisposition;
begin
  stat := FOrb.get_invoke_reply(id, obj, req, ad);
  rec := pull_invoke_orbid(id);
  if rec = nil then exit;
  gstat := orbtypes.NO_EXCEPTION;
  case stat of
    InvokeOk : gstat := orbtypes.NO_EXCEPTION;
    InvokeSysEx: gstat := orbtypes.SYSTEM_EXCEPTION;
    InvokeUserEx: gstat := orbtypes.USER_EXCEPTION;
    InvokeForward: begin
      if id.get_locationforward_perm then
        gstat := orbtypes.LOCATION_FORWARD_PERM
      else
        gstat := orbtypes.LOCATION_FORWARD;
    end;
    InvokeAddrDisp: gstat := orbtypes.NEEDS_ADDRESSING_MODE;
  end;
  outc := TGIOPOutContext.Create(rec.connection.codec, nil);
  if not rec.connection.codec.put_invoke_reply(outc, rec.reqid, gstat, obj, req, ad) then begin
    //outc.reset();
    req.set_out_args(SystemException.Create_type(st_MARSHAL) as IORBException);
    rec.connection.codec.put_invoke_reply(outc, rec.reqid, orbtypes.SYSTEM_EXCEPTION, obj, req, ad);
  end;
  rec.connection.output(outc.buffer);
  del_invoke_orbid(rec);
end;

function TIIOPServer.handle_invoke_request(const conn: IGIOPConnection;
  const inc: IGIOPInContext): Boolean;
var
  obj : IORBObject;
  reqid: _ulong;
  resp_exp : Byte;
  req: IORBRequest;
  msgid: MessageID;
  orbid, orbid2: ORBMessageID;
  rec : IIIOPServerInvokeRecord;
  pr : IPrincipal;
begin
  result := false;
  obj := TORBObject.Create(TIOR.Create as IIOR);
  pr := conn.transport().get_principal;
  if not conn.codec.get_invoke_request(inc,reqid,resp_exp,obj,req,pr) then begin
    conn.activeDeref();
    conn.deref();
    exit;
  end;
  msgid := FORB.new_messageid;
  if Boolean(resp_exp) then
    begin
      orbid := FORB.new_orbid(msgid);
      conn.ref();
      rec := create_invoke();
      rec.init_invoke(conn,reqid,orbid,req,obj);
      add_invoke(rec);
    end;
  conn.activeDeref();
  orbid2 := exec_invoke_request(inc,obj,req,pr,Boolean(resp_exp),conn,orbid);
  Assert((orbid = orbid2) or ((orbid2 = nil) and not Boolean(resp_exp)));
end;

procedure TIIOPServer.handle_locate_reply(msgid: MessageID);
begin
  assert(false,'not implemented');
end;

function TIIOPServer.handle_locate_request(const conn: IGIOPConnection;
  const inc: IGIOPInContext): Boolean;
begin
  result := false;
  assert(false,'not implemented');
end;

function TIIOPServer.has_object(const obj: IORBObject): Boolean;
begin
  result := false;
end;

procedure TIIOPServer.kill_connection(const conn: IGIOPConnection; redo: boolean);
var
  i : integer;
  rec: IIIOPServerInvokeRecord;
  again : Boolean;
  sync: ISynchronized;
begin
  FConnLock.lock();
  try
    conn._AddRef;
    i := FConnections.IndexOf(conn);
    while i >= 0 do
      begin
        FConnections.Delete(i);
        i := FConnections.IndexOf(conn)
      end;
    conn._Release;
  finally
    FConnLock.unlock();
  end; { try/finally }
  conn.terminate();
{$IFDEF USE_IOP_CACHE}
  if FCacheUsed and (FCacheRecord.connection = conn) then
    begin
      FORB.cancel(FCacheRecord.orbid);
      FCacheUsed := false;
    end;
{$ELSE}
  again := true;
  while again do
    begin
      sync := TSynchronized.Create(FMonitor);
      //FORBIdsLock.lock();
      try
        again := False;
        for i := 0 to FORBIds.count - 1 do
          begin
            rec := IIIOPServerInvokeRecord(FORBIds.Items(i));
            if rec.active and (rec.connection = conn) then
              begin
                rec.deactivate();
                abort_invoke_orbid(rec);
                again := True;
                break;
              end;
          end;
      finally
        //FORBIdsLock.unlock();
        sync := nil;
      end; { try/finally }
    end;
  deref_connection(conn);
{$ENDIF}
end;

function TIIOPServer.listen: Boolean;
var
  addr : IInetAddress;
begin
  addr := TInetAddress.Create('0.0.0.0',0,STREAM);
  result := listen_addr(addr,nil);
end;

function TIIOPServer.listen_addr(addr, addr2: IAddress): Boolean;
var
  tserv: ITransportServer;
  prof : IIORProfile;
  mc : IMultiComponent;
begin
  result := false;
  tserv := addr.make_transport_server();
  tserv.create_thread();
  if not tserv.bind(addr) then exit;
  tserv.block(FConnMgr.dispatcher().isblocking());
  tserv.aselect(FConnMgr.dispatcher(), self);
  mc := TMultiComponent.Create;
  prof := tserv.get_address.make_ior_profile('', 1, mc, FIIOPVersion) as IIORProfile;
  FOrb.ior_template.add_profile(prof);
  FTServers.Add(tserv);
  tserv.start();
  prof := nil; tserv := nil; mc := nil;
  result := true;
end;

function TIIOPServer.bind(mid: ORBMessageID; const repoid: string;
  const oid: IObjectTag; const addr: IAddress): Boolean;
begin
  result := false;
end;

function TIIOPServer.invoke(mid: ORBMessageID; const obj: IORBObject;
  const req: IORBRequest; const pr: IPrincipal; response_exp: Boolean): Boolean;
begin
  result := false;
  assert(false);
end;

procedure TIIOPServer.cancel(mid: ORBMessageID);
begin
  assert(false);
end;


//**************************************************************************
// TIIOPServerInvokeRecord
//**************************************************************************
function TIIOPServerInvokeRecord.connection: IGIOPConnection;
begin
  result := FConnection;
end;

procedure TIIOPServerInvokeRecord.clean();
begin
 FConnection := nil;
 FRequest := nil;
 FObject := nil;
 FORBId := nil;
 FREQId := 0;
 FActive := true;
end;

function TIIOPServerInvokeRecord.get_request: IORBRequest;
begin
  result := nil;
  assert(false,'not implemented');
end;

procedure TIIOPServerInvokeRecord.init_invoke(const conn: IGIOPConnection;
  reqid: MessageID; orbid: ORBMessageID; const req: IORBRequest; const obj: IORBObject);
begin
  FConnection:= conn ;
  FORBId := orbid;
  FREQId := reqid;
  FRequest := req;
  FObject:= obj;
  FOrbMsgId := ORB_Instance.get_msgid(orbid);
  FActive := true;
end;

function TIIOPServerInvokeRecord.orbid: ORBMessageID;
begin
  result := FORBId;
end;

function TIIOPServerInvokeRecord.reqid: MessageID;
begin
  result := FReqID;
end;

procedure TIIOPServerInvokeRecord.set_orbid(oid: ORBMessageID);
begin
  //assert(false,'not implemented');
  FORBId := oid;
end;

procedure TIIOPServerInvokeRecord.set_reqid(reqid: MessageID);
begin
  //assert(false,'not implemented');
  FREQId := reqid;
end;

function TIIOPServerInvokeRecord.orbmsgid: MessageID;
begin
  result := FOrbMsgId;
end;

function TIIOPServerInvokeRecord.active: boolean;
begin
  result := FActive;
end;

procedure TIIOPServerInvokeRecord.deactivate;
begin
  FActive := false;
end;

{$ELSE}
implementation
{$ENDIF}
end.
