// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit policies;

interface

uses
  policy_int,orbtypes,orb,orb_int,static,stdstat,code_int,
  exceptions,except_int;

{$IFNDEF USELIB}

type

  TPolicy = class(TORBObject,IPolicy)
  private
    Fpt : PolicyType;
  protected
    function policy_type(): PolicyType;
    function copy(): IPolicy; virtual;
    procedure _destroy();
    // DORB extention
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
  public
    constructor Create(pt: PolicyType);
    class function _narrow(const obj : IORBObject): IPolicy;
  end;

  TThreadPolicy = class(TPolicy,IThreadPolicy)
  private
    FValue : ThreadPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): ThreadPolicyValue;
  public
    constructor Create(pt: PolicyType; val: ThreadPolicyValue);
    class function _narrow(const obj : IORBObject): IThreadPolicy;
  end;

  TLifespanPolicy = class(TPolicy,ILifespanPolicy)
  private
    FValue : LifespanPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): LifespanPolicyValue;
  public
    constructor Create(pt: PolicyType; val: LifespanPolicyValue);
    class function _narrow(const obj : IORBObject): ILifespanPolicy;
  end;

  TIdUniquenessPolicy = class(TPolicy,IIdUniquenessPolicy)
  private
    FValue : IdUniquenessPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): IdUniquenessPolicyValue;
  public
    constructor Create(pt: PolicyType; val: IdUniquenessPolicyValue);
    class function _narrow(const obj : IORBObject): IIdUniquenessPolicy;
  end;

  TIdAssignmentPolicy = class(TPolicy,IIdAssignmentPolicy)
  private
    FValue : IdAssignmentPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): IdAssignmentPolicyValue;
  public
    constructor Create(pt: PolicyType; val: IdAssignmentPolicyValue);
    class function _narrow(const obj : IORBObject): IIdAssignmentPolicy;
  end;

  TImplicitActivationPolicy = class(TPolicy,IImplicitActivationPolicy)
  private
    FValue : ImplicitActivationPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): ImplicitActivationPolicyValue;
  public
    constructor Create(pt: PolicyType; val: ImplicitActivationPolicyValue);
    class function _narrow(const obj : IORBObject): IImplicitActivationPolicy;
  end;

  TServantRetentionPolicy = class(TPolicy,IServantRetentionPolicy)
  private
    FValue : ServantRetentionPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): ServantRetentionPolicyValue;
  public
    constructor Create(pt: PolicyType; val: ServantRetentionPolicyValue);
    class function _narrow(const obj : IORBObject): IServantRetentionPolicy;
  end;

  TRequestProcessingPolicy = class(TPolicy,IRequestProcessingPolicy)
  private
    FValue : RequestProcessingPolicyValue;
  protected
    function copy(): IPolicy; override;
    function narrow_helper(const repoid: string): Pointer; override;
    function repoid(): string; override;
    function value(): RequestProcessingPolicyValue;
  public
    constructor Create(pt: PolicyType; val: RequestProcessingPolicyValue);
    class function _narrow(const obj : IORBObject): IRequestProcessingPolicy;
  end;

  TPolicyError_marshaller = class(TAbstractStat,IStaticTypeInfo)
  protected
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  {** IDL:omg.org/CORBA/PolicyError:1.0 }
  TPolicyError = class(UserException,IPolicyError)
  private
    Freason : TPolicyErrorCode;
  protected
    procedure throw; override;
    function clone(): IORBException; override;
    function repoid(): PChar; override;
    procedure encode(const enc: IEncoder); override;
    procedure reason(const val : TPolicyErrorCode); overload;
  public
    function reason: TPolicyErrorCode; overload;
    constructor Create(ex: TPolicyError); overload;
    constructor Create(_reason: TPolicyErrorCode = 0); overload;
  end;

var
  PolicyError_marshaller : IStaticTypeInfo;

implementation

//*********************************************************************
//  TPolicy
//*********************************************************************
constructor TPolicy.Create(pt: PolicyType);
begin
  inherited Create();
  FPt := pt;
end;

procedure TPolicy._destroy;
begin

end;

function TPolicy.copy: IPolicy;
begin
  result := TPolicy.Create(Fpt);
end;

function TPolicy.policy_type: PolicyType;
begin
  result := Fpt;
end;

class function TPolicy._narrow(const obj: IORBObject): IPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/Policy:1.0');
      if ptr <> nil then result := IPolicy(ptr);
    end;
end;

function TPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/Policy:1.0' then
    result := Pointer(self as IPolicy);
end;

function TPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/Policy:1.0';
end;

//*********************************************************************
//  TThreadPolicy
//*********************************************************************
constructor TThreadPolicy.Create(pt: PolicyType; val: ThreadPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TThreadPolicy._narrow(const obj: IORBObject): IThreadPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/ThreadPolicy:1.0');
      if ptr <> nil then result := IThreadPolicy(ptr);
    end;
end;

function TThreadPolicy.copy: IPolicy;
begin
  result := TThreadPolicy.Create(Fpt,FValue);
end;

function TThreadPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/ThreadPolicy:1.0' then result := Pointer(self as IThreadPolicy);
end;

function TThreadPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/ThreadPolicy:1.0';
end;

function TThreadPolicy.value: ThreadPolicyValue;
begin
  result := FValue;
end;

//*********************************************************************
//  TLifespanPolicy
//*********************************************************************
constructor TLifespanPolicy.Create(pt: PolicyType; val: LifespanPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TLifespanPolicy._narrow(const obj: IORBObject): ILifespanPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/LifespanPolicy:1.0');
      if ptr <> nil then result := ILifespanPolicy(ptr);
    end;
end;

function TLifespanPolicy.copy: IPolicy;
begin
  result := TLifespanPolicy.Create(Fpt,FValue);
end;

function TLifespanPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/LifespanPolicy:1.0' then result := Pointer(self as ILifespanPolicy);
end;

function TLifespanPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/LifespanPolicy:1.0';
end;

function TLifespanPolicy.value: LifespanPolicyValue;
begin
  result := FValue;
end;

//*********************************************************************
//  TIdUniquenessPolicy
//*********************************************************************
constructor TIdUniquenessPolicy.Create(pt: PolicyType;
  val: IdUniquenessPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TIdUniquenessPolicy._narrow(
  const obj: IORBObject): IIdUniquenessPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/IdUniquenessPolicy:1.0');
      if ptr <> nil then result := IIdUniquenessPolicy(ptr);
    end;
end;

function TIdUniquenessPolicy.copy: IPolicy;
begin
  result := TIdUniquenessPolicy.Create(Fpt,FValue);
end;

function TIdUniquenessPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/IdUniquenessPolicy:1.0' then result := Pointer(self as IIdUniquenessPolicy);
end;

function TIdUniquenessPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/IdUniquenessPolicy:1.0';
end;

function TIdUniquenessPolicy.value: IdUniquenessPolicyValue;
begin
  result := FValue
end;

//*********************************************************************
//  TIdAssignmentPolicy
//*********************************************************************
constructor TIdAssignmentPolicy.Create(pt: PolicyType;
  val: IdAssignmentPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TIdAssignmentPolicy._narrow(
  const obj: IORBObject): IIdAssignmentPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/IdAssignmentPolicy:1.0');
      if ptr <> nil then result := IIdAssignmentPolicy(ptr);
    end;
end;

function TIdAssignmentPolicy.copy: IPolicy;
begin
  result := TIdAssignmentPolicy.Create(Fpt,FValue);
end;

function TIdAssignmentPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/IdAssignmentPolicy:1.0' then result := Pointer(self as IIdAssignmentPolicy);
end;

function TIdAssignmentPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/IdAssignmentPolicy:1.0';
end;

function TIdAssignmentPolicy.value: IdAssignmentPolicyValue;
begin
  result := FValue
end;

//*********************************************************************
//  TImplicitActivationPolicy
//*********************************************************************
constructor TImplicitActivationPolicy.Create(pt: PolicyType;
  val: ImplicitActivationPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TImplicitActivationPolicy._narrow(
  const obj: IORBObject): IImplicitActivationPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/ImplicitActivationPolicy:1.0');
      if ptr <> nil then result := IImplicitActivationPolicy(ptr);
    end;
end;

function TImplicitActivationPolicy.copy: IPolicy;
begin
  result := TImplicitActivationPolicy.Create(Fpt,FValue);
end;

function TImplicitActivationPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/ImplicitActivationPolicy:1.0' then result := Pointer(self as IImplicitActivationPolicy);
end;

function TImplicitActivationPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/ImplicitActivationPolicy:1.0';
end;

function TImplicitActivationPolicy.value: ImplicitActivationPolicyValue;
begin
  result := FValue
end;

//*********************************************************************
//  TServantRetentionPolicy
//*********************************************************************
constructor TServantRetentionPolicy.Create(pt: PolicyType;
  val: ServantRetentionPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TServantRetentionPolicy._narrow(
  const obj: IORBObject): IServantRetentionPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/ServantRetentionPolicy:1.0');
      if ptr <> nil then result := IServantRetentionPolicy(ptr);
    end;
end;

function TServantRetentionPolicy.copy: IPolicy;
begin
  result := TServantRetentionPolicy.Create(Fpt,FValue);
end;

function TServantRetentionPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/ServantRetentionPolicy:1.0' then result := Pointer(self as IServantRetentionPolicy);
end;

function TServantRetentionPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/ServantRetentionPolicy:1.0';
end;

function TServantRetentionPolicy.value: ServantRetentionPolicyValue;
begin
  result := FValue
end;

//*********************************************************************
//  TRequestProcessingPolicy
//*********************************************************************
constructor TRequestProcessingPolicy.Create(pt: PolicyType;
  val: RequestProcessingPolicyValue);
begin
  inherited Create(pt);
  FValue := val;
end;

class function TRequestProcessingPolicy._narrow(
  const obj: IORBObject): IRequestProcessingPolicy;
var
  ptr : pointer;
begin
  result := nil;
  if assigned(obj) then
    begin
      ptr := obj.narrow_helper('IDL:omg.org/CORBA/RequestProcessingPolicy:1.0');
      if ptr <> nil then result := IRequestProcessingPolicy(ptr);
    end;
end;

function TRequestProcessingPolicy.copy: IPolicy;
begin
  result := TRequestProcessingPolicy.Create(Fpt,FValue);
end;

function TRequestProcessingPolicy.narrow_helper(const repoid: string): Pointer;
begin
  result := nil;
  if repoid = 'IDL:omg.org/CORBA/RequestProcessingPolicy:1.0' then result := Pointer(self as IRequestProcessingPolicy);
end;

function TRequestProcessingPolicy.repoid: string;
begin
  result := 'IDL:omg.org/CORBA/RequestProcessingPolicy:1.0';
end;

function TRequestProcessingPolicy.value: RequestProcessingPolicyValue;
begin
  result := FValue
end;

//***********************************************************
// TPolicyError_marshaller
//***********************************************************
function TPolicyError_marshaller.demarshal(dec: IDecoder;const addr : Pointer): Boolean;
var
  repoid: string;
  _reason: TPolicyErrorCode;
begin
  result := false;
  if not dec.except_begin(repoid) then exit;
  if not stat_short.demarshal(dec,@_reason) then exit;
  IPolicyError(addr^).reason(_reason);
  if not dec.except_end then exit;
  result := true;
end;

procedure TPolicyError_marshaller.marshal(enc: IEncoder;const addr : Pointer);
var
  _reason: TPolicyErrorCode;
begin
  enc.except_begin('IDL:omg.org/CORBA/PolicyError:1.0');
  _reason:= IPolicyError(addr^).reason;
  stat_short.marshal(enc,@_reason);
  enc.except_end;
end;

procedure TPolicyError_marshaller._free(var addr: Pointer);
begin
  IPolicyError(addr^) := nil;
end;

procedure TPolicyError_marshaller._create(var addr: Pointer);
begin
  IPolicyError(addr^) := TPolicyError.Create();
end;

procedure TPolicyError_marshaller._assign(dst, src: Pointer);
begin
  IPolicyError(dst^) := IPolicyError(src^);
end;

//***********************************************************
// TPolicyError
//***********************************************************
constructor TPolicyError.Create(ex: TPolicyError);
begin
  Freason:= ex.reason;
end;

procedure TPolicyError.throw;
begin
  raise TPolicyError.Create(self);
end;

function TPolicyError.clone(): IORBException;
begin
  result := TPolicyError.Create(self);
end;

procedure TPolicyError.encode(const enc: IEncoder);
var
  int : IPolicyError;
begin
  int := self;
  PolicyError_marshaller.marshal(enc,@int);
end;

constructor TPolicyError.Create(_reason: TPolicyErrorCode);
begin
  Freason := _reason;
end;

function TPolicyError.repoid(): PChar;
begin
  result := 'IDL:omg.org/CORBA/PolicyError:1.0';
end;

procedure TPolicyError.reason(const val: TPolicyErrorCode);
begin
  Freason := val;
end;

function TPolicyError.reason: TPolicyErrorCode;
begin
  result := Freason;
end;

initialization
  PolicyError_marshaller := TPolicyError_marshaller.Create();
{$ELSE}
implementation
{$ENDIF}  
end.

