// **********************************************************************
//
// Copyright (c) 2001 MT Tools.
//
// All Rights Reserved
//
// MT_DORB is based in part on the product DORB,
// written by Shadrin Victor
//
// See Readme.txt for contact information
//
// **********************************************************************
unit std_seq;

interface

uses
  stdstat,code_int,orbtypes;

{$IFNDEF USELIB}

type

  TStat_octet_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_char_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_wchar_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_string_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_wstring_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_any_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_ulong_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_long_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_short_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    procedure _assign(dst,src: Pointer); override;
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

  TStat_boolean_seq = class(TAbstractStat,IStaticTypeInfo)
  private
    FTC: ITypeCode;
  protected
    function demarshal(dec: IDecoder;const addr : Pointer): Boolean;
    procedure marshal(enc: IEncoder; const addr : Pointer);
    function typecode(): ITypeCode; override;
    procedure _create(var addr: Pointer); override;
    procedure _free(var addr: Pointer);
  end;

{$IFDEF LIBRARY}
var
  _stat_seq_octet : IStaticTypeInfo;
  _stat_seq_char : IStaticTypeInfo;
  _stat_seq_wchar : IStaticTypeInfo;
  _stat_seq_string : IStaticTypeInfo;
  _stat_seq_wstring : IStaticTypeInfo;
  _stat_seq_ulong : IStaticTypeInfo;
  _stat_seq_long : IStaticTypeInfo;
  _stat_seq_any : IStaticTypeInfo;
  _stat_seq_short : IStaticTypeInfo;
  _stat_seq_boolean : IStaticTypeInfo;

  function stat_seq_octet : IStaticTypeInfo;
  function stat_seq_char : IStaticTypeInfo;
  function stat_seq_wchar : IStaticTypeInfo;
  function stat_seq_string : IStaticTypeInfo;
  function stat_seq_wstring : IStaticTypeInfo;
  function stat_seq_ulong : IStaticTypeInfo;
  function stat_seq_long : IStaticTypeInfo;
  function stat_seq_any : IStaticTypeInfo;
  function stat_seq_short : IStaticTypeInfo;
  function stat_seq_boolean : IStaticTypeInfo;
{$ELSE}
var
  stat_seq_octet : IStaticTypeInfo;
  stat_seq_char : IStaticTypeInfo;
  stat_seq_wchar : IStaticTypeInfo;
  stat_seq_string : IStaticTypeInfo;
  stat_seq_wstring : IStaticTypeInfo;
  stat_seq_ulong : IStaticTypeInfo;
  stat_seq_long : IStaticTypeInfo;
  stat_seq_any : IStaticTypeInfo;
  stat_seq_short : IStaticTypeInfo;
  stat_seq_boolean : IStaticTypeInfo;
{$ENDIF}


implementation


uses
  any,SysUtils,tcode;

//*********************************************************************
//  TStat_octet_seq
//*********************************************************************
function TStat_octet_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(OctetSeq(addr^),len);
  if not dec.get_octets(Pointer(addr^)^,len) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_octet_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
begin
  len := length(OctetSeq(addr^));
  enc.seq_begin(len);
  enc.put_octets(Pointer(addr^)^,len);
  enc.seq_end;
end;

function TStat_octet_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_octet,0);
  result := FTC;
end;

procedure TStat_octet_seq._create(var addr: Pointer);
begin
  OctetSeq(addr^) := nil;
end;

procedure TStat_octet_seq._free(var addr: Pointer);
begin
  OctetSeq(addr^) := nil;
end;

//*********************************************************************
//  TStat_char_seq
//*********************************************************************
function TStat_char_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(OctetSeq(addr^),len);
  if not dec.get_chars(Pointer(addr^)^,len) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_char_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
begin
  len := length(OctetSeq(addr^));
  enc.seq_begin(len);
  enc.put_chars(Pointer(addr^)^,len);
  enc.seq_end;
end;

function TStat_char_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_char,0);
  result := FTC;
end;

procedure TStat_char_seq._create(var addr: Pointer);
begin
  OctetSeq(addr^) := nil;
end;

procedure TStat_char_seq._free(var addr: Pointer);
begin
  OctetSeq(addr^) := nil;
end;

//*********************************************************************
//  TStat_wchar_seq
//*********************************************************************
function TStat_wchar_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(WCharSeq(addr^),len);
  if not dec.get_wchars(Pointer(addr^)^,len) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_wchar_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
begin
  len := length(WCharSeq(addr^));
  enc.seq_begin(len);
  enc.put_wchars(Pointer(addr^)^,len);
  enc.seq_end;
end;

function TStat_wchar_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_wchar,0);
  result := FTC;
end;

procedure TStat_wchar_seq._create(var addr: Pointer);
begin
  WCharSeq(addr^) := nil;
end;

procedure TStat_wchar_seq._free(var addr: Pointer);
begin
  WCharSeq(addr^) := nil;
end;

//*********************************************************************
//  TStat_string_seq
//*********************************************************************
function TStat_string_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(StringSeq(addr^),len);
  for i := 0 to Pred(len) do
    if not dec.get_string(StringSeq(addr^)[i]) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_string_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
  i : integer;
begin
  len := length(StringSeq(addr^));
  enc.seq_begin(len);
  for i := 0 to Pred(len) do
    enc.put_string(StringSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_string_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_octet,0);
  result := FTC;
end;


procedure TStat_string_seq._create(var addr: Pointer);
begin
  StringSeq(addr^) := nil;
end;

procedure TStat_string_seq._free(var addr: Pointer);
begin
  StringSeq(addr^) := nil;
end;

//*********************************************************************
//  TStat_wstring_seq
//*********************************************************************
function TStat_wstring_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(WStringSeq(addr^),len);
  for i := 0 to Pred(len) do
    if not dec.get_wstring(WStringSeq(addr^)[i]) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_wstring_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
  i : integer;
begin
  len := length(WStringSeq(addr^));
  enc.seq_begin(len);
  for i := 0 to Pred(len) do
    enc.put_string(WStringSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_wstring_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_wchar,0);
  result := FTC;
end;


procedure TStat_wstring_seq._create(var addr: Pointer);
begin
  WStringSeq(addr^) := nil;
end;

procedure TStat_wstring_seq._free(var addr: Pointer);
begin
  WStringSeq(addr^) := nil;
end;

//*********************************************************************
//  TStat_any_seq
//*********************************************************************
function TStat_any_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(AnySeq(addr^),len);
  for i := 0 to Pred(len) do
    //if not dec.get_any(AnySeq(addr^)[i]) then exit;
    if not stat_Any.demarshal(dec, @AnySeq(addr^)[i]) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_any_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
  i : integer;
begin
  len := length(AnySeq(addr^));
  enc.seq_begin(len);
  for i := 0 to Pred(len) do
    enc.put_any(AnySeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_any_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_any,0);
  result := FTC;
end;


procedure TStat_any_seq._create(var addr: Pointer);
begin
  AnySeq(addr^) := nil;
end;

procedure TStat_any_seq._free(var addr: Pointer);
begin
  AnySeq(addr^) := nil;
end;

//*********************************************************************
//  TStat_long_seq
//*********************************************************************
procedure TStat_long_seq._assign(dst, src: Pointer);
begin
  LongSeq(dst^) := System.copy(LongSeq(src^),0,length(LongSeq(src^)));
end;

procedure TStat_long_seq._create(var addr: Pointer);
begin
  LongSeq(addr^) := nil;
end;

procedure TStat_long_seq._free(var addr: Pointer);
begin
  LongSeq(addr^) := nil;
end;

function TStat_long_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  l : _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(l) then exit;
  setLength(LongSeq(addr^),l);
  for i := 0 to pred(l) do
    if not dec.get_long(LongSeq(addr^)[i]) then exit;
  if not dec.seq_end then exit;
  result := true;
end;

procedure TStat_long_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  i : integer;
  l : _ulong;
begin
  l := length(LongSeq(addr^));
  enc.seq_begin(l);
  for i := 0 to Pred(l) do
    enc.put_long(LongSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_long_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_long,0);
  result := FTC;
end;

//*********************************************************************
//  TStat_ulong_seq
//*********************************************************************
procedure TStat_ulong_seq._assign(dst, src: Pointer);
begin
  ULongSeq(dst^) := System.copy(ULongSeq(src^),0,length(ULongSeq(src^)));
end;

procedure TStat_ulong_seq._create(var addr: Pointer);
begin
  ULongSeq(addr^) := nil;
end;

procedure TStat_ulong_seq._free(var addr: Pointer);
begin
  ULongSeq(addr^) := nil;
end;

function TStat_ulong_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  l : _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(l) then exit;
  setLength(ULongSeq(addr^),l);
  for i := 0 to pred(l) do
    if not dec.get_ulong(ULongSeq(addr^)[i]) then exit;
  if not dec.seq_end then exit;
  result := true;
end;

procedure TStat_ulong_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  i : integer;
  l : _ulong;
begin
  l := length(ULongSeq(addr^));
  enc.seq_begin(l);
  for i := 0 to Pred(l) do
    enc.put_ulong(ULongSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_ulong_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_ulong,0);
  result := FTC;
end;

//*********************************************************************
//  TStat_short_seq
//*********************************************************************
procedure TStat_short_seq._assign(dst, src: Pointer);
begin
  ShortSeq(dst^) := System.copy(ShortSeq(src^),0,length(ShortSeq(src^)));
end;

procedure TStat_short_seq._create(var addr: Pointer);
begin
  ShortSeq(addr^) := nil;
end;

procedure TStat_short_seq._free(var addr: Pointer);
begin
  ShortSeq(addr^) := nil;
end;

function TStat_short_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  l : _ulong;
  i : integer;
begin
  result := false;
  if not dec.seq_begin(l) then exit;
  setLength(ShortSeq(addr^),l);
  for i := 0 to pred(l) do
    if not dec.get_short(ShortSeq(addr^)[i]) then exit;
  if not dec.seq_end then exit;
  result := true;
end;

procedure TStat_short_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  i : integer;
  l : _ulong;
begin
  l := length(ShortSeq(addr^));
  enc.seq_begin(l);
  for i := 0 to Pred(l) do
    enc.put_short(ShortSeq(addr^)[i]);
  enc.seq_end;
end;

function TStat_short_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_short,0);
  result := FTC;
end;

//*********************************************************************
//  TStat_boolean_seq
//*********************************************************************
function TStat_boolean_seq.demarshal(dec: IDecoder;
  const addr: Pointer): Boolean;
var
  len: _ulong;
begin
  result := false;
  if not dec.seq_begin(len) then exit;
  setLength(OctetSeq(addr^),len);
  if not dec.get_octets(Pointer(addr^)^,len) then exit;
  if not dec.seq_end() then exit;
  result := true;
end;

procedure TStat_boolean_seq.marshal(enc: IEncoder; const addr: Pointer);
var
  len: _ulong;
begin
  len := length(OctetSeq(addr^));
  enc.seq_begin(len);
  enc.put_octets(Pointer(addr^)^,len);
  enc.seq_end;
end;

function TStat_boolean_seq.typecode: ITypeCode;
begin
  if FTC = nil then
    FTC := TTypeCode.create_sequence_tc(_tc_octet,0);
  result := FTC;
end;

procedure TStat_boolean_seq._create(var addr: Pointer);
begin
  OctetSeq(addr^) := nil;
end;

procedure TStat_boolean_seq._free(var addr: Pointer);
begin
  OctetSeq(addr^) := nil;
end;

{$IFDEF LIBRARY}
function stat_seq_octet : IStaticTypeInfo;
begin
  result := _stat_seq_octet
end;

function stat_seq_char : IStaticTypeInfo;
begin
  result := _stat_seq_char
end;

function stat_seq_wchar : IStaticTypeInfo;
begin
  result := _stat_seq_wchar
end;

function stat_seq_string : IStaticTypeInfo;
begin
  result := _stat_seq_string
end;

function stat_seq_wstring : IStaticTypeInfo;
begin
  result := _stat_seq_wstring
end;

function stat_seq_ulong : IStaticTypeInfo;
begin
  result := _stat_seq_ulong
end;

function stat_seq_long : IStaticTypeInfo;
begin
  result := _stat_seq_long
end;

function stat_seq_any : IStaticTypeInfo;
begin
  result := _stat_seq_any
end;

function stat_seq_short : IStaticTypeInfo;
begin
  result := _stat_seq_short
end;

function stat_seq_boolean : IStaticTypeInfo;
begin
  result := _stat_seq_boolean
end;

exports
  stat_seq_octet,
  stat_seq_char,
  stat_seq_wchar,
  stat_seq_string,
  stat_seq_wstring,
  stat_seq_ulong,
  stat_seq_long,
  stat_seq_any,
  stat_seq_short,
  stat_seq_boolean;
initialization
  _stat_seq_octet := TStat_octet_seq.Create();
  _stat_seq_char := TStat_char_seq.Create();
  _stat_seq_wchar := TStat_wchar_seq.Create();
  _stat_seq_string := TStat_string_seq.Create();
  _stat_seq_wstring := TStat_wstring_seq.Create();
  _stat_seq_ulong := TStat_ulong_seq.Create;
  _stat_seq_long := TStat_long_seq.Create;
  _stat_seq_any := TStat_any_seq.Create;
  _stat_seq_short := TStat_short_seq.Create;
  _stat_seq_boolean := TStat_boolean_seq.Create;
{$ELSE}
initialization
  stat_seq_octet := TStat_octet_seq.Create();
  stat_seq_char := TStat_char_seq.Create();
  stat_seq_wchar := TStat_wchar_seq.Create();
  stat_seq_string := TStat_string_seq.Create();
  stat_seq_wstring := TStat_wstring_seq.Create();
  stat_seq_ulong := TStat_ulong_seq.Create;
  stat_seq_long := TStat_long_seq.Create;
  stat_seq_any := TStat_any_seq.Create;
  stat_seq_short := TStat_short_seq.Create;
  stat_seq_boolean := TStat_boolean_seq.Create;
{$ENDIF}
{$ELSE}
  function stat_seq_octet : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_char : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_wchar : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_string : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_wstring : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_ulong : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_long : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_any : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_short : IStaticTypeInfo; external MTDORB_Library_Name;
  function stat_seq_boolean : IStaticTypeInfo; external MTDORB_Library_Name;
implementation
{$ENDIF}
end.
